#!/usr/bin/env python
# -*- coding: utf-8 -*-

#   _______   _____   _____    _
#  |__   __| |_   _| |  __ \  | |
#     | |      | |   | |__) | | |
#     | |      | |   |  _  /  | |
#     | |     _| |_  | | \ \  | |____
#     |_|    |_____| |_|  \_\ |______|
#
# Copyright (C) 2018-2020 University of Oxford
# Part of the FMRIB Software Library (FSL)
# Author: Istvan N. Huszar


# FMRIB Software Library, Release 6.0.4 (c) 2020, The University of Oxford
# (the "Software")
# 
# The Software remains the property of the University of Oxford ("the
# University").
# 
# The Software is distributed "AS IS" under this Licence solely for
# non-commercial use in the hope that it will be useful, but in order
# that the University as a charitable foundation protects its assets for
# the benefit of its educational and research purposes, the University
# makes clear that no condition is made or to be implied, nor is any
# warranty given or to be implied, as to the accuracy of the Software,
# or that it will be suitable for any particular purpose or for use
# under any specific conditions. Furthermore, the University disclaims
# all responsibility for the use which is made of the Software. It
# further disclaims any liability for the outcomes arising from using
# the Software.
# 
# The Licensee agrees to indemnify the University and hold the
# University harmless from and against any and all claims, damages and
# liabilities asserted by third parties (including claims for
# negligence) which arise directly or indirectly from the use of the
# Software or the sale of any products based on the Software.
# 
# No part of the Software may be reproduced, modified, transmitted or
# transferred in any form or by any means, electronic or mechanical,
# without the express permission of the University. The permission of
# the University is not required if the said reproduction, modification,
# transmission or transference is done without financial return, the
# conditions of this Licence are imposed upon the receiver of the
# product, and all original and amended source code is included in any
# transmitted product. You may be held legally responsible for any
# copyright infringement that is caused or encouraged by your failure to
# abide by these terms and conditions.
# 
# You are not permitted under this Licence to use this Software
# commercially. Use for which any financial return is received shall be
# defined as commercial use, and includes (1) integration of all or part
# of the source code or the Software into a product for sale or license
# by or on behalf of Licensee to third parties or (2) use of the
# Software or any derivative of it for research with the final aim of
# developing software products for sale or license to a third party or
# (3) use of the Software or any derivative of it for research with the
# final aim of developing non-software products for sale or license to a
# third party, or (4) use of the Software to provide any service to an
# external organisation for which payment is received. If you are
# interested in using the Software commercially, please contact Oxford
# University Innovation ("OUI"), the technology transfer company of the
# University, to negotiate a licence. Contact details are:
# Innovation@innovation.ox.ac.uk quoting reference BS/9564.
# 


import unittest
import numpy as np

import tirl
from tirl.beta import beta_function, BetaClassMeta
from tirl.timage import TImage
from tirl.tirlobject import TIRLObject


class TestImage(unittest.TestCase):

    def test_testimg(self):
        img = tirl.testimg()
        self.assertIsInstance(img, TImage, "Test Image must be a TImage.")
        self.assertGreaterEqual(img.vdim, 2, "Image dimensions must be >=2.")
        self.assertTrue(np.any(np.abs(img.data) > 0),
                        "Test Image must not be all-zeros.")


@beta_function
def func_that_is_beta():
    return True


class BetaClass(TIRLObject, metaclass=BetaClassMeta):
    def __init__(self):
        super(BetaClass, self).__init__()
        self.value = True


class TestBetaMarkers(unittest.TestCase):

    @beta_function
    def method_that_is_beta(self):
        return True

    def test_beta_function(self):
        warnchkmsg = "Beta function should display a warning."
        with self.assertWarns(UserWarning, msg=warnchkmsg):
            retval = func_that_is_beta()
        self.assertTrue(
            retval, "Beta function must be executed despite the warning.")

    def test_beta_method(self):
        warnchkmsg = "Beta method should display a warning."
        with self.assertWarns(UserWarning, msg=warnchkmsg):
            retval = func_that_is_beta()
        self.assertTrue(
            retval, "Beta method must be executed despite the warning.")

    def test_beta_class(self):
        warnchkmsg = "Beta class should display a warning on instantiation."
        with self.assertWarns(UserWarning, msg=warnchkmsg):
            instance = BetaClass()
        self.assertIsInstance(instance, BetaClass, "Object must be created.")
        self.assertTrue(instance.value, "Object attributes must be the same.")


if __name__ == '__main__':
    unittest.main()
